package cn.sh.stc.sict.theme.hpgp.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.theme.hpgp.dao.HpHzjlMapper;
import cn.sh.stc.sict.theme.hpgp.model.HpHzjl;
import cn.sh.stc.sict.theme.hpgp.model.HpgpBusyIdlePrediction;
import cn.sh.stc.sict.theme.hpgp.model.HpgpKsHot;
import cn.sh.stc.sict.theme.hpgp.service.HpHzjlService;
import cn.sh.stc.sict.theme.hpgp.service.HpgpBusyIdlePredictionService;
import cn.sh.stc.sict.theme.hpgp.service.HpgpDepartmentRankService;
import cn.sh.stc.sict.theme.hpgp.service.HpgpKsHotService;
import cn.sh.stc.sict.theme.hphy.model.HpDeptInfo;
import cn.sh.stc.sict.theme.hphy.model.HpDocInfo;
import cn.sh.stc.sict.theme.hphy.service.HpDeptInfoService;
import cn.sh.stc.sict.theme.hphy.service.HpDocInfoService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * (HpHzjl)表服务实现类
 *
 * @author F_xh
 * @since 2022-08-15 14:14:33
 */
@AllArgsConstructor
@Service("hpHzjlService")
public class HpHzjlServiceImpl extends ServiceImpl<HpHzjlMapper, HpHzjl> implements HpHzjlService {

    private final HpgpDepartmentRankService hpgpDepartmentRankService;
    private final HpgpBusyIdlePredictionService hpgpBusyIdlePredictionService;
    private final HpDocInfoService hpDocInfoService;
    private final HpgpKsHotService hpgpKsHotService;
    private final HpDeptInfoService hpDeptInfoService;

    private static final Integer MAX_DEPT_SIZE = 3;

    /**
     * 根据身份证号查找最后三次不同科室就诊记录，并计算忙闲
     * 如果未找到，则推送三个热门科室
     *
     * @param certId       身份证号
     * @param hospitalCode 医疗机构代码（医院编码）
     * @return
     */
    @Override
    public List<HpgpBusyIdlePrediction> listByCertIdAndHosCode(String certId,
                                                               String hospitalCode) {
        List<HpgpBusyIdlePrediction> result = new ArrayList<>();
        int resultDeptSize = 0;
        DateTime startTime = DateUtil.tomorrow();
        DateTime endTime = DateUtil.offsetDay(startTime, 6);

        LambdaQueryWrapper<HpHzjl> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(HpHzjl::getZjhm, certId)
                .eq(StrUtil.isNotBlank(hospitalCode), HpHzjl::getYljgdm, hospitalCode)
                .groupBy(HpHzjl::getJzksbm)
                .orderByDesc(HpHzjl::getJzksrq)
                .last("limit " + MAX_DEPT_SIZE);
        List<HpHzjl> recordList = this.list(wrapper);
        if (CollUtil.isNotEmpty(recordList)) {
            for (HpHzjl record : recordList) {
                // 查找科室
                LambdaQueryWrapper<HpDeptInfo> deptWrapper = Wrappers.<HpDeptInfo>lambdaQuery();
                deptWrapper.eq(HpDeptInfo::getHosOrgCode, record.getYljgdm())
                        .eq(HpDeptInfo::getDeptCode, record.getJzksbm())
                        .eq(HpDeptInfo::getDeptLevel, 2);
                HpDeptInfo dept = hpDeptInfoService.getOne(deptWrapper);
                if (ObjectUtil.isNull(dept)) {
                    continue;
                }
                // 忙闲预测
                result.addAll(hpgpBusyIdlePredictionService.listByDept(dept, startTime, endTime));
                resultDeptSize++;
            }
        }

        // 历史科室查询结果少于3个时，采用热点科室数据补充
        if (resultDeptSize < MAX_DEPT_SIZE) {
            List<HpgpKsHot> hotList = hpgpKsHotService.listHotDept(hospitalCode, MAX_DEPT_SIZE - resultDeptSize);
            hotList.forEach(hot -> {
                HpDeptInfo dept = new HpDeptInfo();
                dept.setHosOrgCode(hot.getHospitalCode());
                dept.setHosName(hot.getHospitalName());
                dept.setOneDeptCode(hot.getOneDeptCode());
                dept.setDeptCode(hot.getDeptCode());
                dept.setDeptName(hot.getDeptName());
                result.addAll(hpgpBusyIdlePredictionService.listByDept(dept, startTime, endTime));
            });
        }
        return result;
    }

    @Override
    public List<HpDocInfo> listDoctorByCertIdAndHosCode(String certId,
                                                        String hospitalCode) {
        LambdaQueryWrapper<HpHzjl> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(HpHzjl::getZjhm, certId)
                .eq(StrUtil.isNotBlank(hospitalCode), HpHzjl::getYljgdm, hospitalCode)
                .groupBy(HpHzjl::getYljgdm, HpHzjl::getJzksbm, HpHzjl::getZzysgh)
                .orderByDesc(HpHzjl::getJzksrq)
                .last("limit 10");
        List<HpHzjl> recordList = this.list(wrapper);
        List<HpDocInfo> doctorList = new ArrayList<>();
        if (CollUtil.isNotEmpty(recordList)) {
            recordList.forEach(r -> {
                LambdaQueryWrapper<HpDocInfo> dw = Wrappers.lambdaQuery();
                dw.eq(HpDocInfo::getHosOrgCode, r.getYljgdm())
                        .eq(HpDocInfo::getResourceCode, r.getZzysgh())
                        .eq(HpDocInfo::getResourceName, r.getZzysxm())
                        .last("limit 1");
                HpDocInfo one = hpDocInfoService.getOne(dw);
                if (ObjectUtil.isNotNull(one)) {
                    doctorList.add(one);
                }
            });
        }
        return doctorList;
    }
}
