package cn.sh.stc.sict.theme.hphy.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.dto.WDUserCardInfo;
import cn.sh.stc.sict.theme.hphy.dao.HpPatientCardMapper;
import cn.sh.stc.sict.theme.hphy.model.HpPatientCard;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientBase;
import cn.sh.stc.sict.theme.hphy.service.HpPatientCardService;
import cn.sh.stc.sict.theme.hphy.wd.OrderInfo;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 黄浦-患者-卡号(HpPatientCard)表服务实现类
 *
 * @author F_xh
 * @since 2022-07-14 11:00:02
 */
@Service("hpPatientCardService")
public class HpPatientCardServiceImpl extends ServiceImpl<HpPatientCardMapper, HpPatientCard> implements HpPatientCardService {

    @Override
    public void save(HphyPatientBase base, List<WDUserCardInfo> cardList) {
        if (CollUtil.isNotEmpty(cardList)) {
            cardList.forEach(wdCard -> {
                HpPatientCard card = BeanUtil.toBean(wdCard, HpPatientCard.class);
                card.setUserId(base.getUserId());
                card.setPatientId(base.getId());
                this.save(card);
            });
        }
    }

    @Override
    public List<HpPatientCard> listByUserId(Long userId) {
        LambdaQueryWrapper<HpPatientCard> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(HpPatientCard::getUserId, userId);
        return this.list(wrapper);
    }

    @Override
    public void updateByApp(HphyPatientBase patient, OrderInfo order) {
        if (StrUtil.isNotBlank(order.getMediCardId()) && StrUtil.isNotBlank(order.getMediCardType())) {
            HpPatientCard card = new HpPatientCard();
            card.setCardNo(order.getMediCardId());
            card.setCardType(order.getMediCardType());
            card.setUserId(patient.getUserId());
            card.setPatientId(patient.getId());
            LambdaQueryWrapper<HpPatientCard> wrapper = Wrappers.lambdaQuery();
            wrapper.eq(HpPatientCard::getUserId, patient.getUserId())
                    .eq(HpPatientCard::getPatientId, patient.getId())
                    .eq(HpPatientCard::getCardType, order.getMediCardType());
            this.saveOrUpdate(card, wrapper);
        }
    }
}